#include "filecheck.h"

FileCheck::FileCheck(QObject *parent) : QObject(parent)
{

}

bool FileCheck::isLegalIconFile(const QString &filePath)
{
    QSvgRenderer renderer(filePath);

    if (renderer.isValid()){
        QSize svgSize = renderer.defaultSize();
        if (svgSize.width() >= 96 && svgSize.height() >= 96 && svgSize.width() == svgSize.height()){
            return true;
        }
    }

    return false;
}

bool FileCheck::isLegalWallPaperFile(const QString& filePath, const QString& type)
{
    QFileInfo fileInfo(filePath);
    QImage image(filePath);
    QSize imageSize = image.size();
    qint64 fileSize = fileInfo.size();

    if (type == "wallpaper") {
        if (imageSize.width() != 3840 || imageSize.height() != 2160){
            QMessageBox::information(nullptr, tr("error"), tr("The wallpaper size must be 3840x2160"));
            return false;
        }
        qint64 maxSize = 10 * 1024 * 1024; // 10 MB
        if (fileSize > maxSize){
            QMessageBox::information(nullptr, tr("error"), tr("The file size cannot exceed 10MB"));
            return false;
        }
    } else if (type == "cover") {
        if (imageSize.width() != 1640 || imageSize.height() != 1080){
            QMessageBox::information(nullptr, tr("error"), tr("Cover size must be 1640x1080"));
            return false;
        }
        qint64 maxSize = 5 * 1024 * 1024; // 5 MB
        if (fileSize > maxSize){
            QMessageBox::information(nullptr, tr("error"), tr("The file size cannot exceed 5MB"));
            return false;
        }
    }

    return true;
}

