#include "maininterface.h"



/**
 * @brief MainInterface 构造函数
 *
 * MainInterface 的构造函数，用于初始化界面控件和建立信号连接。
 *
 * @param parent 父控件指针
 */
MainInterface::MainInterface(QWidget *parent) : QWidget(parent)
{
//    this->setBackgroundRole(QPalette::Base);

    m_belowwidget = new QWidget(this);

    init();

    QVBoxLayout *layout = new QVBoxLayout();
    layout->addWidget(m_belowwidget);

    this->setLayout(layout);
    connect(m_globalthemewidget, &GlobalThemeWidget::newWallpaperFilePath, this, [&](const QString& path) {
        emit wallpaperPathChanged(path);
    });
    connect(m_globalthemewidget, &GlobalThemeWidget::newCoverFilePath, this, [&](const QString& path) {
        emit coverPathChanged(path);
    });
    connect(m_globalthemewidget, &GlobalThemeWidget::newRadius, this, [&](const int radius) {
        emit radiusChanged(radius);
    });
    connect(m_globalthemewidget, &GlobalThemeWidget::newAccentColor, this, [&](const QColor selectedColor) {
        emit accentColorChanged(selectedColor);
    });
    connect(m_globalthemewidget, &GlobalThemeWidget::newExterior, this, [&](const QColor selectedColor) {
        emit exteriorChanged(selectedColor);
    });
    connect(m_globalthemewidget, &GlobalThemeWidget::newTransparency, this, [&](const int transparency) {
        emit transparencyChanged(transparency);
    });
    connect(m_globalthemewidget, &GlobalThemeWidget::newWindowRadius, this, [&](const int windowFilletSize) {
        emit windowRadiusChanged(windowFilletSize);
    });
    connect(m_iconthemewidget, &IconThemeWidget::newAppIconsMap, this, [&]( QMap<QString, QString> *appiconsmaps) {
        emit appIconsMapChanged(appiconsmaps);
    });
    connect(m_iconthemewidget, &IconThemeWidget::newSystemIconsMap, this, [&]( QMap<QString, QString> *systemiconsmaps) {
        emit systemIconsMapChanged(systemiconsmaps);
    });
    connect(m_cursorthemewidget, &CursorThemeWidget::newCursorMap, this, [&]( QMap<QString, QString> *cursormaps) {
        emit cursorMapChanged(cursormaps);
    });
    connect(m_cursorthemewidget, &CursorThemeWidget::newTimeCursorMap, this, [&]( QMap<QString, QString> *timecursormaps) {
        emit timeCursorMapChanged(timecursormaps);
    });
    connect(m_plymouththemewidget, &PlymouthThemeWidget::newPlymouthFilePath, this, [&](const QString& path) {
        emit plymouthPathChanged(path);
    });
    connect(m_grubthemewidget, &GrubThemeWidget::newGrubFilePath, this, [&](const QString& path) {
        emit grubPathChanged(path);
    });
}

/**
 * @brief 设置是否启用历史记录
 *
 * 根据给定的布尔值设置是否启用历史记录。
 *
 * @param ishistory 是否启用历史记录
 */
void MainInterface::isHistory(bool ishistory)
{
    m_ishistory = ishistory;
}

/**
 * @brief 初始化主界面
 *
 * 初始化主界面，包括左侧栏、主题堆叠小部件等。
 */
void MainInterface::init()
{

    initLeftBar();

    m_themestackedwidget = new QStackedWidget(m_belowwidget);

    m_globalthemewidget = new GlobalThemeWidget(m_themestackedwidget);
    m_iconthemewidget = new IconThemeWidget(m_ishistory,m_themestackedwidget);
    m_cursorthemewidget = new CursorThemeWidget(m_themestackedwidget);
    m_plymouththemewidget = new PlymouthThemeWidget(m_themestackedwidget);
    m_grubthemewidget = new GrubThemeWidget(m_themestackedwidget);

    connect(m_globalthemewidget,&GlobalThemeWidget::wallpaperupdate,this, &MainInterface::wallpaperupdate);
    connect(this, &MainInterface::wallpaperupdate, m_cursorthemewidget, &CursorThemeWidget::wallpaperupdate);
    connect(this, &MainInterface::wallpaperupdate, m_iconthemewidget, &IconThemeWidget::wallpaperupdate);

    m_themestackedwidget->addWidget(m_globalthemewidget);
    m_themestackedwidget->addWidget(m_iconthemewidget);
    m_themestackedwidget->addWidget(m_cursorthemewidget);
    m_themestackedwidget->addWidget(m_plymouththemewidget);
    m_themestackedwidget->addWidget(m_grubthemewidget);

    m_themestackedwidget->setCurrentIndex(0);

    m_layout = new QHBoxLayout();
    m_layout->addWidget(m_navigationwidget);
    m_layout->addWidget(m_themestackedwidget);

    m_belowwidget->setLayout(m_layout);

    connect(m_globalbtn,&TypeButton::clicked,m_themestackedwidget,[=](){
        m_themestackedwidget->setCurrentIndex(0);
    });
    connect(m_iconbtn,&TypeButton::clicked,m_themestackedwidget,[=](){
        m_themestackedwidget->setCurrentIndex(1);
    });
    connect(m_cursorbtn,&TypeButton::clicked,m_themestackedwidget,[=](){
        m_themestackedwidget->setCurrentIndex(2);
    });
    connect(m_plymouthbtn,&TypeButton::clicked,m_themestackedwidget,[=](){
        m_themestackedwidget->setCurrentIndex(3);
    });
    connect(m_grubbtn,&TypeButton::clicked,m_themestackedwidget,[=](){
        m_themestackedwidget->setCurrentIndex(4);
        m_grubbtn->setChecked(true);
    });
    InfoCreateWidget *info = new InfoCreateWidget(nullptr,"create");
    connect(m_buildbtn,&QPushButton::clicked,info,[=](){
        FileProcess getInfo;
        info->getFileInfo(getInfo.FileSearch(FileProcess::g_date));
        info->show();
        connect(info,&InfoCreateWidget::readytoBuild,this,&MainInterface::startCopy);
        connect(info,&InfoCreateWidget::gohomesignals,this,[=](){
            //发送返回主界面信号给mainwindow
            emit gohomesignals();
        });
    });

}

/**
 * @brief 初始化左侧工具栏
 *
 * 创建一个QWidget作为左侧工具栏，并设置其最小和最大尺寸。然后创建一个QVBoxLayout来管理左侧工具栏的布局。
 * 创建一个QButtonGroup来管理多个按钮的选中状态。将多个按钮添加到按钮组中，并设置它们为可选中状态。
 * 将全局按钮和构建按钮添加到布局中，并设置构建按钮为普通按钮。最后将布局中的控件添加到左侧工具栏中。
 */
void MainInterface::initLeftBar()
{
    m_navigationwidget = new QWidget(m_belowwidget);
    m_navigationwidget->setMinimumSize(200,620);
    m_navigationwidget->setMaximumWidth(200);
    QVBoxLayout *leftLayout = new QVBoxLayout(m_navigationwidget);

    QButtonGroup *buttonGroup = new QButtonGroup(m_navigationwidget);
    buttonGroup->setExclusive(true);
//    QButtonGroup *bootbuttonGroup = new QButtonGroup(m_navigationwidget);

    m_globalbtn = new TypeButton(m_navigationwidget);
    m_globalbtn->setText(tr("GlobalTheme"));
    m_iconbtn = new TypeButton(m_navigationwidget);
    m_iconbtn->setText(tr("IconTheme"));
    m_cursorbtn = new TypeButton(m_navigationwidget);
    m_cursorbtn->setText(tr("CursorTheme"));
    m_plymouthbtn = new TypeButton(m_navigationwidget);
    m_plymouthbtn->setText(tr("PlymouthTheme"));
    m_grubbtn = new TypeButton(m_navigationwidget);
    m_grubbtn->setText(tr("GrubTheme"));

    m_buildbtn = new QPushButton(m_navigationwidget);
    m_buildbtn->setText(tr("Start Building"));

    buttonGroup->addButton(m_globalbtn);
    buttonGroup->addButton(m_iconbtn);
    buttonGroup->addButton(m_cursorbtn);
    buttonGroup->addButton(m_plymouthbtn);
    buttonGroup->addButton(m_grubbtn);

    m_globalbtn->setCheckable(true);
    m_iconbtn->setCheckable(true);
    m_cursorbtn->setCheckable(true);
    m_plymouthbtn->setCheckable(true);
    m_grubbtn->setCheckable(true);

    leftLayout->addWidget(m_globalbtn);
    leftLayout->addWidget(m_iconbtn);
    leftLayout->addWidget(m_cursorbtn);
    leftLayout->addWidget(m_plymouthbtn);
    leftLayout->addWidget(m_grubbtn);
    leftLayout->addStretch();
    leftLayout->addWidget(m_buildbtn);
}

void MainInterface::setGlobalTheme()
{
    m_themestackedwidget->setCurrentIndex(0);
    m_globalbtn->setHidden(false);
    m_globalbtn->setChecked(true);
    m_globalbtn->setHidden(false);
    m_iconbtn->setHidden(false);
    m_cursorbtn->setHidden(false);
    m_plymouthbtn->setHidden(false);
    m_grubbtn->setHidden(false);
    FileProcess::g_createThemeType = "globalTheme";
}

void MainInterface::setIconTheme()
{
    m_themestackedwidget->setCurrentIndex(1);
    m_globalbtn->setHidden(true);
    m_iconbtn->setHidden(false);
    m_iconbtn->setChecked(true);
    m_cursorbtn->setHidden(true);
    m_plymouthbtn->setHidden(true);
    m_grubbtn->setHidden(true);
    FileProcess::g_createThemeType = "iconTheme";
}

void MainInterface::setCursorTheme()
{
    m_themestackedwidget->setCurrentIndex(2);
    m_globalbtn->setHidden(true);
    m_iconbtn->setHidden(true);
    m_cursorbtn->setHidden(false);
    m_cursorbtn->setChecked(true);
    m_plymouthbtn->setHidden(true);
    m_grubbtn->setHidden(true);
    FileProcess::g_createThemeType = "cursorTheme";

}

void MainInterface::setPlymouthTheme()
{
    m_themestackedwidget->setCurrentIndex(3);
    m_globalbtn->setHidden(true);
    m_iconbtn->setHidden(true);
    m_cursorbtn->setHidden(true);
    m_plymouthbtn->setHidden(false);
    m_plymouthbtn->setChecked(true);
    m_grubbtn->setHidden(true);
    FileProcess::g_createThemeType = "plymouthTheme";
}

void MainInterface::setGrubTheme()
{
    m_themestackedwidget->setCurrentIndex(4);
    m_globalbtn->setHidden(true);
    m_iconbtn->setHidden(true);
    m_cursorbtn->setHidden(true);
    m_plymouthbtn->setHidden(true);
    m_grubbtn->setHidden(false);
    m_grubbtn->setChecked(true);
    FileProcess::g_createThemeType = "grubTheme";
}

void MainInterface::getThemeInfo(const HistoryInfo &InfoData)
{
    if (InfoData.themetype == "globalTheme"){
        setGlobalTheme();
        //发送信号，将info传入widget，并且显示在widget
        m_globalthemewidget->eidtInitWidget(InfoData);
    }else if(InfoData.themetype == "cursorTheme"){
        setCursorTheme();
        m_cursorthemewidget->eidtInitWidget(InfoData);
    }else if(InfoData.themetype == "iconTheme"){
        setIconTheme();
        m_iconthemewidget->eidtInitWidget(InfoData);
    }

}

/**
 * @brief 获取应用程序图标映射表
 *
 * 返回一个 QMap<QString, QString> 类型的指针，该指针指向应用程序图标映射表。
 *
 * @return 应用程序图标映射表指针
 */
QMap<QString, QString>* MainInterface::getAppIconsMap()
{
    //资源文件的appiconmap
    return m_iconthemewidget->getAppIconsMap();
}

QMap<QString, QString> *MainInterface::getSystemIconsMap()
{
    //资源文件的systemiconmap
    return m_iconthemewidget->getSystemIconsMap();
}

QMap<QString, QString> *MainInterface::getCurosrMap()
{
    //资源文件的cursoriconmap
    return m_cursorthemewidget->getCursorMap();
}

QMap<QString, QString> *MainInterface::getTimeCurosrMap()
{
    //资源文件的timecursoriconmap
    return m_cursorthemewidget->getTimeCursorMap();
}

/**
 * @brief 刷新界面
 *
 * 根据传入的主题类型刷新界面。
 *
 * @param themeType 主题类型
 */
void MainInterface::refresh(QString themeType)
{
    if(themeType == "global"){
        m_globalthemewidget->refresh();
    } else if(themeType == "icon"){
        m_iconthemewidget->refresh();
    } else if(themeType == "cursor"){
        m_cursorthemewidget->refresh();
    }/* else if(themeType == "plymouth"){
    +        m_plymouththemewidget->refresh();
    +    } else if(themeType == "grub"){
    +        m_grubthemewidget->refresh();
    +    }*/




}

void MainInterface::updateButtonGroup(bool isGlobalTheme)
{
    if(isGlobalTheme){
        m_plymouthbtn->setVisible(false);
        m_grubbtn->setVisible(false);
        m_globalbtn->setVisible(true);
        m_iconbtn->setVisible(true);
        m_cursorbtn->setVisible(true);
    }else{
        m_globalbtn->setVisible(false);
        m_iconbtn->setVisible(false);
        m_cursorbtn->setVisible(false);
        m_plymouthbtn->setVisible(true);
        m_grubbtn->setVisible(true);
    }

}

/**
 * @brief 更新缩略图
 *
 * 将指定的缩略图路径设置为 PlymouthThemeWidget 的缩略图图标。
 *
 * @param thumbnailPath 缩略图路径
 */
void MainInterface::updateThumbnail(QString thumbnailPath)
{
    m_plymouththemewidget->setThumbnailIcon(thumbnailPath);
}

