#include "mainwindow.h"
#include <QUrl>
#include <QDesktopServices>
MainWindow::MainWindow(QWidget *parent)
    : QMainWindow(parent)
{
    setWindowFlags(Qt::FramelessWindowHint);
    m_bridge = Bridge::getInstance();
    FileProcess();


    initUI();
    m_titlebar = new TitleBar(this);
    QVBoxLayout *layout = new QVBoxLayout(this);
    layout->addWidget(m_titlebar);
    layout->addWidget(mainWidget);
    QWidget *centralWidget = new QWidget(this);
    centralWidget->setLayout(layout);
    setCentralWidget(centralWidget);
    setMouseTracking(true);
    connect(m_titlebar, &TitleBar::gohomesignal, this, &MainWindow::onGoHomeClicked);
    connect(m_maininterface, &MainInterface::wallpaperPathChanged, m_bridge, &Bridge::wallpaperPathChanged);
    connect(m_maininterface, &MainInterface::coverPathChanged, m_bridge, &Bridge::coverPathChanged);
    connect(m_maininterface, &MainInterface::radiusChanged, m_bridge, &Bridge::radiusChanged);
    connect(m_maininterface, &MainInterface::accentColorChanged, m_bridge, &Bridge::accentColorChanged);
    connect(m_maininterface, &MainInterface::exteriorChanged, m_bridge, &Bridge::exteriorChanged);
    connect(m_maininterface, &MainInterface::transparencyChanged, m_bridge, &Bridge::transparencyChanged);
    connect(m_maininterface, &MainInterface::windowRadiusChanged, m_bridge, &Bridge::windowRadiusChanged);
//    connect(m_maininterface, &MainInterface::appIconsMapChanged, this,[=]{
//        m_bridge->updateIconCache(m_maininterface->getAppIconsMap(),"appicon");
//    });
    connect(m_maininterface, &MainInterface::appIconsMapChanged, m_bridge, &Bridge::appIconsMapChanged);
    connect(m_maininterface, &MainInterface::systemIconsMapChanged, m_bridge, &Bridge::systemIconsMapChanged);
    connect(m_maininterface, &MainInterface::cursorMapChanged, m_bridge, &Bridge::cursorMapChanged);
    connect(m_maininterface, &MainInterface::timeCursorMapChanged, m_bridge, &Bridge::timeCursorMapChanged);
    connect(m_maininterface, &MainInterface::plymouthPathChanged, m_bridge, &Bridge::plymouthPathChanged);
    connect(m_maininterface, &MainInterface::grubPathChanged, m_bridge, &Bridge::grubPathChanged);
//    connect(m_maininterface, &MainInterface::startCopy, m_bridge, &Bridge::startCopy);
    connect(m_bridge,&Bridge::updateInfo,m_historywidget,&HistoryWidget::updateInfo);
    connect(m_bridge,&Bridge::updateThumbnail,m_maininterface,&MainInterface::updateThumbnail);

    connect(m_maininterface, &MainInterface::gohomesignals, this, &MainWindow::onGoHomeClicked);
}

MainWindow::~MainWindow()
{
}

/**
 * @brief 初始化界面
 */
void MainWindow::initUI()
{
    QPalette palette;
    palette.setColor(QPalette::Background, Qt::white);
    this->setAutoFillBackground(true);
    this->setPalette(palette);

    this->setMinimumSize(1096, 640);

    m_maininterface = new MainInterface();

    m_entrywidget = new QWidget();
    initEntry();

    QVBoxLayout* alllayout = new QVBoxLayout;


    QGridLayout* layout = new QGridLayout;
    layout->addWidget(m_globalbtn,0,0);
    layout->addWidget(m_iconbtn,0,1);
    layout->addWidget(m_cursorbtn,0,2);

    layout->addWidget(m_plymouthbtn,1,0);
    layout->addWidget(m_grubbtn,1,1);

    m_guideWidget = new QWidget();

    QHBoxLayout *labellayout = new QHBoxLayout();
    QLabel *Label = new QLabel(m_guideWidget);
    Label->setText(tr("kylin-theme-builder"));
//    QPushButton *guideLabel = new QPushButton(m_guideWidget);
    QLabel *guideLabel = new QLabel(m_guideWidget);
    QString str = "<a href=\"https://gitee.com/openkylin/community/tree/master/sig/UKUITheme\">" + tr("User guide") + "</a>";
    guideLabel->setText(str);
    guideLabel->setTextFormat(Qt::RichText);
    guideLabel->setCursor(Qt::PointingHandCursor);
    guideLabel->setOpenExternalLinks(true); // 将此属性设置为 true

    labellayout->addStretch(1);
    labellayout->addWidget(Label);
    labellayout->addWidget(guideLabel);
    labellayout->addStretch(1);

    m_guideWidget->setLayout(labellayout);

    m_historywidget = new HistoryWidget();
    m_historywidget->setMinimumHeight(2000);
    QDir builderDir = QDir::homePath()+"/.cache/theme-build/";
    if(!builderDir.exists()|| builderDir.isEmpty()){
        alllayout->addStretch(1);
        alllayout->addLayout(layout);
        alllayout->addStretch(1);
        alllayout->addWidget(m_guideWidget);
        alllayout->addStretch(1);
    }else{
        alllayout->addStretch(1);
        alllayout->addLayout(layout);
        alllayout->addStretch(1);
        alllayout->addWidget(m_guideWidget);
        alllayout->addWidget(m_historywidget);
    }


    m_entrywidget->setLayout(alllayout);

    QScrollArea* scrollArea = new QScrollArea;
    scrollArea->setWidgetResizable(true);
    scrollArea->setWidget(m_entrywidget);

    m_stackedWidget = new QStackedWidget(this);
    m_stackedWidget->addWidget(scrollArea);
    m_stackedWidget->addWidget(m_maininterface);

    QVBoxLayout* mainLayout = new QVBoxLayout;
    mainLayout->addWidget(m_stackedWidget);

    mainWidget = new QWidget;
    mainWidget->setLayout(mainLayout);
    QButtonGroup* buttonGroup = new QButtonGroup(this);
    buttonGroup->addButton(m_globalbtn);
    buttonGroup->addButton(m_iconbtn);
    buttonGroup->addButton(m_cursorbtn);
    buttonGroup->addButton(m_plymouthbtn);
    buttonGroup->addButton(m_grubbtn);

    connect(m_historywidget,&HistoryWidget::startSecondEdit,[=](HistoryInfo InfoData){
        m_maininterface->isHistory(true);
        m_stackedWidget->setCurrentWidget(m_maininterface);

        m_bridge->createFileManager(InfoData.historytime);

        m_maininterface->getThemeInfo(InfoData);
    });

    connect(buttonGroup, QOverload<QAbstractButton*>::of(&QButtonGroup::buttonClicked), [=](QAbstractButton* button) {
        m_stackedWidget->setCurrentWidget(m_maininterface);
        m_maininterface->isHistory(false);
        if (button == m_globalbtn) {
            m_maininterface->setGlobalTheme();
            m_fileProcess.FileCreate("globalTheme");
            m_bridge->createFileManager(FileProcess::g_date);
            m_bridge->updateIconCache(m_maininterface->getAppIconsMap(),"appicon");
            m_bridge->updateIconCache(m_maininterface->getSystemIconsMap(),"systemicon");
            m_bridge->updateIconCache(m_maininterface->getCurosrMap(),"cursor");
            m_bridge->updateIconCache(m_maininterface->getTimeCurosrMap(),"timecursor");

            m_bridge->createConfig();
            m_bridge->createGrub();
            m_bridge->createPlymouth();

            m_maininterface->refresh("global");
        } else if (button == m_iconbtn) {
            m_maininterface->setIconTheme();
            m_fileProcess.FileCreate("iconTheme");
            m_bridge->createFileManager(FileProcess::g_date);
            m_bridge->updateIconCache(m_maininterface->getAppIconsMap(),"appicon");
            m_bridge->updateIconCache(m_maininterface->getSystemIconsMap(),"systemicon");
            m_maininterface->refresh("icon");

        } else if (button == m_cursorbtn) {
            m_maininterface->setCursorTheme();
            m_fileProcess.FileCreate("cursorTheme");
            m_bridge->createFileManager(FileProcess::g_date);
            m_bridge->updateIconCache(m_maininterface->getCurosrMap(),"cursor");
            m_bridge->updateIconCache(m_maininterface->getTimeCurosrMap(),"timecursor");
            m_maininterface->refresh("cursor");

        }else if (button == m_plymouthbtn) {
            m_maininterface->setPlymouthTheme();
            m_fileProcess.FileCreate("plymouthTheme");
            m_bridge->createFileManager(FileProcess::g_date);
            m_bridge->createPlymouth();

        }else if (button == m_grubbtn) {
            m_maininterface->setGrubTheme();
            m_fileProcess.FileCreate("grubTheme");
            m_bridge->createFileManager(FileProcess::g_date);
            m_bridge->createGrub();

        }

        if (m_historywidget == nullptr) {
            m_historywidget = new HistoryWidget();
            m_historywidget->setMinimumHeight(2000);
            alllayout->removeWidget(m_guideWidget);
            alllayout->addWidget(m_guideWidget);
            alllayout->addWidget(m_historywidget);
        }
    });

    QFileSystemWatcher* watcher = new QFileSystemWatcher(this);
    watcher->addPath(builderDir.absolutePath());
    connect(watcher, &QFileSystemWatcher::directoryChanged, this, [=](const QString& path) {
        if (!builderDir.exists() || builderDir.isEmpty()) {
            alllayout->removeWidget(m_guideWidget);
            alllayout->removeItem(alllayout->itemAt(alllayout->count()-1));
            delete m_historywidget;
            m_historywidget = nullptr;
            alllayout->addStretch(1);
            alllayout->addWidget(m_guideWidget);
            alllayout->addStretch(1);
        }
    });
}

/**
 * @brief 初始化入口
 *
 * 初始化各个按钮的入口，并设置相应的图标和提示信息。
 */
void MainWindow::initEntry()
{
    m_globalbtn = new EntryButton();
    m_iconbtn = new EntryButton();
    m_cursorbtn = new EntryButton();
    m_plymouthbtn = new EntryButton();
    m_grubbtn = new EntryButton();

    QIcon icon = QIcon(QPixmap(":/resource/background/homepage-global.png"));
    m_globalbtn->setEntry(icon,tr("Global Theme"));
    m_globalbtn->setTip(tr("Topics include basic styles for windows and controls, icons, cursors, and more!"));
    QIcon icon1 = QIcon(QPixmap(":/resource/background/homepage-icons.png"));
    m_iconbtn->setEntry(icon1,tr("Icon Theme"));
    m_iconbtn->setTip(tr("The icon theme includes multiple application icons and system settings homepage icons."));
    QIcon icon2 = QIcon(QPixmap(":/resource/background/homepage-cursor.png"));
    m_cursorbtn->setEntry(icon2,tr("Cursor Theme"));
    m_cursorbtn->setTip(tr("Cursor theme for pointer cursor icon"));
    QIcon icon3 = QIcon(QPixmap(":/resource/background/homepage-plymouth.png"));
    m_plymouthbtn->setEntry(icon3,tr("Plymouth Theme"));
    m_plymouthbtn->setTip(tr("Boot animation can be customized to boot screen effect"));
    QIcon icon4 = QIcon(QPixmap(":/resource/background/homepage-grub.png"));
    m_grubbtn->setEntry(icon4,tr("GRUB Theme"));
    m_grubbtn->setTip(tr("Modify GURB background wallpaper"));
}

void MainWindow::refresh()
{
//    m_maininterface->refresh();
}

/**
 * @brief 点击"GoHome"按钮后执行的函数
 *
 * 当用户点击"GoHome"按钮后，执行该函数。
 */
void MainWindow::onGoHomeClicked()
{
    //返回主界面判断，是否在主界面点击返回按钮
    if (FileProcess::g_date != nullptr){
        //是否存在改动
        if(CursorThemeWidget::g_themeChange || IconThemeWidget::g_themeChange || PlymouthThemeWidget::g_themeChange ||
                GrubThemeWidget::g_themeChange || GlobalThemeWidget::g_themeChange){
            //创建询问是否保存界面
            cacheConfirmedWidget *m_cacheConfirmedWidget = new cacheConfirmedWidget();
            m_cacheConfirmedWidget->show();
            //根据询问界面返回信号处理历史记录
            connect(m_cacheConfirmedWidget, &cacheConfirmedWidget::cacheClean, this, [=](){
                m_stackedWidget->setCurrentIndex(0);
                //返回主界面前 清除全局变量
                FileProcess::g_createThemeType = nullptr;
                FileProcess::g_date = nullptr;
                FileProcess::g_themeENName = nullptr;
                CursorThemeWidget::g_themeChange = false;
                IconThemeWidget::g_themeChange = false;
                PlymouthThemeWidget::g_themeChange = false;
                GrubThemeWidget::g_themeChange = false;
                GlobalThemeWidget::g_themeChange = false;
            });
            connect(m_cacheConfirmedWidget, &cacheConfirmedWidget::cacheSave, this, [=](){
                m_historywidget->updateHistoryDir();
                m_stackedWidget->setCurrentIndex(0);
                FileProcess::g_createThemeType = nullptr;
                FileProcess::g_date = nullptr;
                FileProcess::g_themeENName = nullptr;
                CursorThemeWidget::g_themeChange = false;
                IconThemeWidget::g_themeChange = false;
                PlymouthThemeWidget::g_themeChange = false;
                GrubThemeWidget::g_themeChange = false;
                GlobalThemeWidget::g_themeChange = false;
            });
        } else {
            //无修改时，删除时间戳目录
            FileProcess m_fileClean;
            m_fileClean.FileClean(FileProcess::g_date);
            FileProcess::g_createThemeType = nullptr;
            FileProcess::g_date = nullptr;
            FileProcess::g_themeENName = nullptr;
            CursorThemeWidget::g_themeChange = false;
            IconThemeWidget::g_themeChange = false;
            PlymouthThemeWidget::g_themeChange = false;
            GrubThemeWidget::g_themeChange = false;
            GlobalThemeWidget::g_themeChange = false;
            m_stackedWidget->setCurrentIndex(0);
        }
    } else {
        m_stackedWidget->setCurrentIndex(0);
    }
//    refresh();
}

/**
 * @brief 设置入口按钮的图标和文本
 *
 * 设置入口按钮的图标和文本。
 *
 * @param icon QIcon 对象，表示要设置的图标
 * @param text QString 对象，表示要设置的文本
 */
void EntryButton::setEntry(const QIcon &icon, const QString &text)
{
    QFont font("Arial", 18);
    font.setBold(true);
    textLabel->setFont(font);

    iconLabel->setPixmap(icon.pixmap(320, 320));
    textLabel->setText(text);
    textLabel->setAlignment(Qt::AlignCenter);
    textLabel->setFont(QFont("Arial" , 18));
}

/**
 * @brief 设置提示文字
 *
 * 设置 EntryButton 控件的提示文字。
 *
 * @param text 提示文字
 */
void EntryButton::setTip(const QString &text)
{
    tipLabel->setText(text);
    QFont font;
    font.setPointSize(10);
    tipLabel->setFont(font);
    tipLabel->setText(text);
    tipLabel->setWordWrap(true);
}

/**
 * @brief 构造函数
 *
 * 创建一个 EntryButton 对象，继承自 QPushButton。
 *
 * @param parent 父对象指针
 */
EntryButton::EntryButton(QWidget *parent): QPushButton(parent)
{
    this->setFixedSize(240,280);
    QVBoxLayout *layout = new QVBoxLayout;
    layout->setContentsMargins(0, 0, 0, 0);
    setLayout(layout);

    iconLabel = new QLabel;
    textLabel = new QLabel;
    tipLabel = new QLabel;

    layout->addWidget(iconLabel);
    layout->addWidget(textLabel);
    layout->addWidget(tipLabel);
}

/**
 * @brief 鼠标按下事件处理函数
 *
 * 当鼠标左键按下时，执行该函数。
 *
 * @param event 鼠标事件指针
 */
void MainWindow::mousePressEvent(QMouseEvent *event)
{
    // 判断是否按下左键
    if (event->button() == Qt::LeftButton) {
        // 设置拖动标志为真
        this->m_drag = true;
        // 记录按下位置的坐标
        this->dragPos = event->pos();
        // 记录全局位置
        this->resizeDownPos = event->globalPos();
        // 记录按下时窗口的矩形大小
        this->mouseDownRect = this->rect();
    }
}

/**
 * @brief 鼠标移动事件
 *
 * 当鼠标移动时，该函数会被调用。
 *
 * @param event 鼠标事件指针
 */
void MainWindow::mouseMoveEvent(QMouseEvent * event)
{
    if (resizeRegion != Default)
    {
        // 处理调整窗体大小的操作
        handleResize();
        return;
    }
    if(m_move) {
        // 移动窗体
        move(event->globalPos() - dragPos);
        return;
    }
    QPoint clientCursorPos = event->pos();
    QRect r = this->rect();
    QRect resizeInnerRect(resizeBorderWidth, resizeBorderWidth, r.width() - 2*resizeBorderWidth, r.height() - 2*resizeBorderWidth);
    if(r.contains(clientCursorPos) && !resizeInnerRect.contains(clientCursorPos)) { //调整窗体大小
        // 获取窗体的可调整大小区域
        ResizeRegion resizeReg = getResizeRegion(clientCursorPos);
        // 设置鼠标的形状为可调整大小形状
        setResizeCursor(resizeReg);
        if (m_drag && (event->buttons() & Qt::LeftButton)) {
            // 设置窗体的可调整大小区域，并处理调整大小操作
            resizeRegion = resizeReg;
            handleResize();
        }
    }
    else { //移动窗体
        // 设置鼠标的形状为箭头形状
        setCursor(Qt::ArrowCursor);
        if (m_drag && (event->buttons() & Qt::LeftButton)) {
            // 设置标志位为移动窗体状态，并移动窗体
            m_move = true;
            move(event->globalPos() - dragPos);
        }
    }
}

/**
 * @brief 鼠标释放事件
 *
 * 当鼠标释放时触发本事件。
 *
 * @param event QMouseEvent 指针，指向发生事件的对象。
 */
void MainWindow::mouseReleaseEvent(QMouseEvent *event)
{
    m_drag = false;
    if(m_move) {
        m_move = false;
        handleMove(event->globalPos()); //鼠标放开后若超出屏幕区域自动吸附于屏幕顶部/左侧/右侧
    }
    resizeRegion = Default;
    setCursor(Qt::ArrowCursor);
}

/**
 * @brief 设置调整大小的游标
 *
 * 根据给定的调整区域设置对应的游标。
 *
 * @param region 调整区域
 */
void MainWindow::setResizeCursor(ResizeRegion region)
{
    switch (region)
    {
    case North:
    case South:
        // 设置上下调整光标
        setCursor(Qt::SizeVerCursor);
        break;
    case East:
    case West:
        // 设置左右调整光标
        setCursor(Qt::SizeHorCursor);
        break;
    case NorthWest:
    case SouthEast:
        // 设置对角线调整光标（从左上到右下）
        setCursor(Qt::SizeFDiagCursor);
        break;
    default:
        // 设置对角线调整光标（从右上到左下）
        setCursor(Qt::SizeBDiagCursor);
        break;
    }
}

/**
 * @brief 获取窗口调整大小区域
 *
 * 根据给定的客户端位置获取窗口调整大小的区域。
 *
 * @param clientPos 客户端位置
 *
 * @return 返回窗口调整大小区域枚举值
 */
ResizeRegion MainWindow::getResizeRegion(QPoint clientPos)
{
    // 如果客户点的y坐标小于等于边框宽度
    if (clientPos.y() <= resizeBorderWidth) {
        // 如果客户点的x坐标小于等于边框宽度
        if (clientPos.x() <= resizeBorderWidth)
            // 返回北西方向
            return NorthWest;
        // 如果客户点的x坐标大于等于窗口宽度减去边框宽度
        else if (clientPos.x() >= this->width() - resizeBorderWidth)
            // 返回北东方向
            return NorthEast;
        else
            // 返回北方向
            return North;
    }
    // 如果客户点的y坐标大于等于窗口高度减去边框宽度
    else if (clientPos.y() >= this->height() - resizeBorderWidth) {
        // 如果客户点的x坐标小于等于边框宽度
        if (clientPos.x() <= resizeBorderWidth)
            // 返回南西方向
            return SouthWest;
        // 如果客户点的x坐标大于等于窗口宽度减去边框宽度
        else if (clientPos.x() >= this->width() - resizeBorderWidth)
            // 返回南东方向
            return SouthEast;
        else
            // 返回南方向
            return South;
    }
    else {
        // 如果客户点的x坐标小于等于边框宽度
        if (clientPos.x() <= resizeBorderWidth)
            // 返回西方向
            return West;
        else
            // 返回东方向
            return East;
    }
}

/**
 * @brief 处理移动事件
 *
 * 当移动事件发生时，该函数用于处理移动事件，并调整窗口位置。
 *
 * @param pt 鼠标当前位置
 */
void MainWindow::handleMove(QPoint pt)
{
    QDesktopWidget* desktop = QApplication::desktop(); // 创建 QDesktopWidget 实例
    QPoint currentPos = pt - dragPos;
    if(currentPos.x()<desktop->x()) { //吸附于屏幕左侧
        currentPos.setX(desktop->x());
    }
    else if (currentPos.x()+this->width()>desktop->width()) { //吸附于屏幕右侧
        currentPos.setX(desktop->width()-this->width());
    }
    if(currentPos.y()<desktop->y()) { //吸附于屏幕顶部
        currentPos.setY(desktop->y());
    }
    move(currentPos);
}

/**
 * @brief 处理窗口大小调整
 *
 * 根据鼠标的位置和移动距离，调整窗口的大小。
 */
void MainWindow::handleResize()
{
    // 计算x轴和y轴的偏移量
    int xdiff = QCursor::pos().x() - resizeDownPos.x();
    int ydiff = QCursor::pos().y() - resizeDownPos.y();

    // 根据不同的resizeRegion进行不同的操作
    switch (resizeRegion)
    {
        case East:
        {
            // 东方向，宽度增加
            resize(mouseDownRect.width()+xdiff, this->height());
            break;
        }
        case West:
        {
            // 西方向，宽度减少
            resize(mouseDownRect.width()-xdiff, this->height());
            // 移动到原始位置，x轴偏移量增加
            move(resizeDownPos.x()+xdiff, this->y());
            break;
        }
        case South:
        {
            // 南方向，高度增加
            resize(this->width(),mouseDownRect.height()+ydiff);
            break;
        }
        case North:
        {
            // 北方向，高度减少
            resize(this->width(),mouseDownRect.height()-ydiff);
            // 移动到原始位置，y轴偏移量增加
            move(this->x(), resizeDownPos.y()+ydiff);
            break;
        }
        case SouthEast:
        {
            // 东南方向，宽度和高度都增加
            resize(mouseDownRect.width() + xdiff, mouseDownRect.height() + ydiff);
            break;
        }
        case NorthEast:
        {
            // 东北方向，宽度增加，高度减少
            resize(mouseDownRect.width()+xdiff, mouseDownRect.height()-ydiff);
            // 移动到原始位置，x轴偏移量增加，y轴偏移量增加
            move(this->x(), resizeDownPos.y()+ydiff);
            break;
        }
        case NorthWest:
        {
            // 西北方向，宽度减少，高度减少
            resize(mouseDownRect.width()-xdiff, mouseDownRect.height()-ydiff);
            // 移动到原始位置，x轴偏移量减少，y轴偏移量减少
            move(resizeDownPos.x()+xdiff, resizeDownPos.y()+ydiff);
            break;
        }
        case SouthWest:
        {
            // 西南方向，宽度减少，高度增加
            resize(mouseDownRect.width()-xdiff, mouseDownRect.height()+ydiff);
            // 移动到原始位置，x轴偏移量减少，y轴偏移量增加
            move(resizeDownPos.x()+xdiff, this->y());
            break;
        }
    }
}


