#include "globalthemewidget.h"
#include "../fileProcess/filecheck.h"

bool GlobalThemeWidget::g_themeChange;
/**
 * @brief 构造函数
 *
 * 创建一个 GlobalThemeWidget 对象，并设置父窗口。
 *
 * @param parent 父窗口指针
 */
GlobalThemeWidget::GlobalThemeWidget(QWidget *parent) : QWidget(parent)
{
    initPreviewWidget();
    initRightWidget();
    m_globalthemelayout = new QHBoxLayout();
    m_globalthemelayout->addWidget(m_previewwidget);
    m_globalthemelayout->addWidget(m_rightwidget);
    this->setLayout(m_globalthemelayout);
}

/**
 * @brief 获取壁纸路径
 *
 * 返回壁纸的路径。
 *
 * @return 壁纸路径
 */
QString GlobalThemeWidget::getWallpaperPath()
{
    return m_wallpaperpath;
}

/**
 * @brief 初始化小部件
 *
 * 根据给定的历史信息编辑小部件的初始化。
 *
 * @param InfoData 历史信息
 */
void GlobalThemeWidget::eidtInitWidget(const HistoryInfo &InfoData)
{
    m_info = InfoData;
    m_info.isEdit = true;
    this->setRadiusSetting(m_info.radius);
    this->setAccentColor(m_info.accentcolor);
    this->setCover(m_info.coverpath);
    this->setWallpaper(m_info.wallpaperpath);
    this->setTransparency(m_info.transparency);
    this->setWindowRadius(m_info.windowradius);
    //    this->update();
}

/**
 * @brief 刷新全局主题界面
 *
 * 刷新全局主题界面，更新设置项。
 */
void GlobalThemeWidget::refresh()
{

    this->setRadiusSetting(6);
    this->setAccentColor("#3790FA");
    this->setWindowRadius(6);
    this->setTransparency(65);
    this->setCover(":/resource/background/background-glazing.png");
    this->setWallpaper(":/resource/background/background-light.png");
    m_preview->updatescale1();
}

/**
 * @brief 初始化预览小部件
 *
 * 创建一个新的 QWidget 作为预览小部件，并设置其大小和背景颜色。
 * 预览小部件用于展示壁纸的预览图。
 */
void GlobalThemeWidget::initPreviewWidget()
{
    m_previewwidget = new QWidget(this);
    m_previewwidget->setMinimumSize(495,620);

    QPalette palette;
    QColor bgColor("#F5F5F5");
    palette.setColor(QPalette::Window, bgColor);
    m_previewwidget->setPalette(palette);
    m_previewwidget->setAutoFillBackground(true);

    m_preview = new GlobalImageWidget(m_previewwidget,m_wallpaperpath);

    m_previewlayout = new QVBoxLayout(m_previewwidget);
    addspaceritem();
    m_previewwidget->setLayout(m_previewlayout);
}

/**
 * @brief 初始化右侧小部件
 *
 * 初始化右侧小部件，包括滚动区域、视口小部件、各个主题设置小部件等。
 */
void GlobalThemeWidget::initRightWidget()
{
    m_rightwidget = new QWidget(this);
    m_rightwidget->setMinimumSize(400,620);
    m_rightwidget->setMaximumWidth(400);
    m_rightwidget->setBackgroundRole(QPalette::Base);

    m_scrollArea = new QScrollArea(m_rightwidget);
    m_scrollArea->setMinimumSize(m_rightwidget->size());
    m_scrollArea->setWidgetResizable(true);

    m_viewportwidget = new QWidget(m_scrollArea);
    m_scrollArea->setWidget(m_viewportwidget);
    QVBoxLayout *mainLayout = new QVBoxLayout(m_viewportwidget);


    initCoverWidget();
    initWallPaperWidget();
    initExteriorWidget();
    initFilletWidget();
    initAccentColorWidget();
    initTransparencyWidget();
    initWindowFilletWidget();

    mainLayout->addWidget(m_coverwidget);
    mainLayout->addStretch(1);
    mainLayout->addWidget(m_wallpaperwidget);
    mainLayout->addStretch(1);
    mainLayout->addWidget(m_exteriorwidget);
    mainLayout->addStretch(1);
    mainLayout->addWidget(m_filletwidget);
    mainLayout->addStretch(1);
    mainLayout->addWidget(m_accentcolorwidget);
    mainLayout->addStretch(1);
    mainLayout->addWidget(m_transparencywidget);
    mainLayout->addStretch(1);
    mainLayout->addWidget(m_windowfilletwidget);
    mainLayout->addStretch(1);

    m_viewportwidget->setLayout(mainLayout);

    QVBoxLayout *mainWidgetLayout = new QVBoxLayout(m_rightwidget);
    mainWidgetLayout->addWidget(m_scrollArea);
    m_rightwidget->setLayout(mainWidgetLayout);

}

/**
 * @brief 初始化CoverWidget
 *
 * 初始化CoverWidget控件，并设置相关属性。
 */
void GlobalThemeWidget::initCoverWidget()
{
    m_coverwidget = new QWidget(m_viewportwidget);
    m_coverwidget->setFixedHeight(182);
    QLabel *title = new QLabel(m_coverwidget);
    if(m_info.isEdit){
        title->setText("111111111111111111111111");
    }else{
        title->setText(tr("Overall theme rendering"));
    }

    QVBoxLayout *layout = new QVBoxLayout();

    QWidget *cover = new QWidget(m_coverwidget);
    cover->setFixedHeight(160);
    QHBoxLayout *coverlayout = new QHBoxLayout(cover);
    m_coverbtn = new QPushButton(cover);
    m_coverbtn->setFixedSize(160,100);
    QLabel *tip = new QLabel(cover);

    QPixmap pixmap(m_coverpath);
    m_coverbtn->setIcon(QIcon(pixmap));
    m_coverbtn->setIconSize(m_coverbtn->size());

    tip->setText("<html>尺寸：1640*1080<br>大小：不超过 5 MB<br>格式：PNG</html>");

    coverlayout->addWidget(m_coverbtn);
    coverlayout->addWidget(tip);
    cover->setLayout(coverlayout);

    connect(m_coverbtn, &QPushButton::clicked, this, [=]() {
        QString newFilePath = QFileDialog::getOpenFileName(this, tr("Select picture file"), "", tr("Picture file (*.png *.jpg)"));
        if (newFilePath.isEmpty()) {
            newFilePath = m_coverpath;
        }else{
            if(!FileCheck::isLegalWallPaperFile(newFilePath,"cover")){
                newFilePath = m_coverpath;
            }
        }
        m_coverpath = newFilePath;
        m_coverbtn->setIcon(QIcon(m_coverpath));
        m_coverbtn->setIconSize(m_coverbtn->size());
        emit newCoverFilePath(newFilePath);
        g_themeChange = true;

    });

    layout->addStretch(1);
    layout->addWidget(title);
    layout->addWidget(cover);
    m_coverwidget->setLayout(layout);

}

/**
 * @brief 初始化桌面壁纸小部件
 *
 * 初始化桌面壁纸小部件，并设置相关属性。
 */
void GlobalThemeWidget::initWallPaperWidget()
{
    m_wallpaperwidget = new QWidget(m_viewportwidget);
    m_wallpaperwidget->setFixedHeight(182);
    QLabel *title = new QLabel(m_wallpaperwidget);
    title->setText(tr("Desktop wallpaper"));
    QVBoxLayout *layout = new QVBoxLayout();

    QWidget *cover = new QWidget(m_wallpaperwidget);
    cover->setFixedHeight(160);
    QHBoxLayout *coverlayout = new QHBoxLayout(cover);
    wallpaperbtn = new QPushButton(cover);
    wallpaperbtn->setFixedSize(160,100);
    QLabel *tip = new QLabel(cover);
    tip->setText("<html>尺寸：3840*2160<br>大小：不超过 10 MB<br>格式：PNG</html>");

    coverlayout->addWidget(wallpaperbtn);
    coverlayout->addWidget(tip);
    cover->setLayout(coverlayout);

    QPixmap pixmap(m_wallpaperpath);
    wallpaperbtn->setIcon(QIcon(pixmap));
    wallpaperbtn->setIconSize(wallpaperbtn->size());

    connect(wallpaperbtn, &QPushButton::clicked, this, [=]() {
        QString newFilePath = QFileDialog::getOpenFileName(this, tr("Select picture file"), "", tr("Picture file (*.png *.jpg)"));

        if (!newFilePath.isEmpty()) {
            if(FileCheck::isLegalWallPaperFile(newFilePath,"wallpaper")){
                updateWallpaperFilePath(newFilePath);
                emit wallpaperupdate(newFilePath);
                emit newWallpaperFilePath(newFilePath);

                QPixmap pixmap(newFilePath);
                wallpaperbtn->setIcon(QIcon(pixmap));
                wallpaperbtn->setIconSize(wallpaperbtn->size());
                g_themeChange = true;
            }
        } else {
            qDebug() << "Selected file path:" << newFilePath;
            updateWallpaperFilePath(m_wallpaperpath);
        }
    });

    layout->addStretch(1);
    layout->addWidget(title);
    layout->addWidget(cover);
    m_wallpaperwidget->setLayout(layout);
}

/**
 * @brief 初始化外观小部件
 *
 * 创建一个新的 QWidget，并设置其固定高度为 65。
 * 创建一个 QLabel，设置其宽度与小部件相同，并设置文本为 "Window appearance"。
 * 创建一个 QHBoxLayout，并添加 QLabel 和 QComboBox 到布局中。
 * 设置 QComboBox 的项目，包括 "light" 和 "dark" 两个选项，分别对应白色和黑色。
 * 连接 QComboBox 的 currentIndexChanged 信号到槽函数，当选项改变时更新预览小部件的背景颜色和覆盖图像，并发送 newExterior 信号。
 */
void GlobalThemeWidget::initExteriorWidget()
{
    m_exteriorwidget = new QWidget(m_viewportwidget);
    m_exteriorwidget->setFixedHeight(65);
    QLabel *title = new QLabel(m_exteriorwidget);
    title->setFixedWidth(m_exteriorwidget->width());
    title->setText(tr("Window appearance"));
    QHBoxLayout *layout = new QHBoxLayout();
    QComboBox *combobox = new QComboBox(m_exteriorwidget);

    combobox->addItem(tr("light"),QColor(255, 255, 255));
    combobox->addItem(tr("dark"), QColor(29, 29, 29));
    connect(combobox, QOverload<int>::of(&QComboBox::currentIndexChanged), this, [=](int index){
        QColor selectedColor = combobox->itemData(index).value<QColor>();
        QString overlayImage;

        if (index == 0) {
            overlayImage = ":/resource/background/panel-light.png";
        } else if (index == 1) {
            overlayImage = ":/resource/background/panel-dark.png";
        }

        m_preview->updateWidgetBackgroundColor(selectedColor);
        m_preview->updateOverlayImage(overlayImage);
        m_preview->updatescale();

        emit newExterior(selectedColor);
        g_themeChange = true;
    });

    layout->addWidget(title);
    layout->addWidget(combobox);

    m_exteriorwidget->setLayout(layout);
}

/**
 * @brief 初始化圆角控件小部件
 *
 * 初始化圆角控件小部件，并设置相关属性和布局。
 */
void GlobalThemeWidget::initFilletWidget()
{
    m_filletwidget = new QWidget(m_viewportwidget);
    m_filletwidget->setFixedHeight(65);
    QLabel *title = new QLabel(m_filletwidget);
    title->setFixedWidth(m_filletwidget->width());
    title->setText(tr("Control fillet"));
    QHBoxLayout *layout = new QHBoxLayout();
    m_filletslider = new QSlider(m_filletwidget);

    m_filletslider->setOrientation(Qt::Horizontal);
    m_filletslider->setFixedWidth(210);
    m_filletslider->setRange(0,18);
    m_filletslider->setSingleStep(1);
    m_filletslider->setTickInterval(1);
    m_filletslider->setFixedWidth(120);
    m_filletslider->setValue(6);

    QPushButton *showButton = new QPushButton(m_filletwidget);
    showButton->setDisabled(true);
    showButton->setFixedSize(60,36);
    int value = 0;
    value = m_filletslider->value();
    showButton->setText(QString::number(m_filletslider->value())+"px");

    QObject::connect(m_filletslider, &QSlider::valueChanged, [=](int value) {
        showButton->setText(QString::number(value)+"px");
        m_preview->updatescale();
        m_preview->updateControlRadius(value);

        emit newRadius(value);
        g_themeChange = true;
    });

    layout->addWidget(title);
    layout->addStretch(1);
    layout->addWidget(m_filletslider);
    layout->addWidget(showButton);

    m_filletwidget->setLayout(layout);
}

/**
 * @brief 初始化强调色控件
 *
 * 初始化强调色控件，并设置相应的布局和功能。
 */
void GlobalThemeWidget::initAccentColorWidget()
{
    m_accentcolorwidget = new QWidget(m_viewportwidget);
    m_accentcolorwidget->setFixedHeight(65);
    QLabel *title = new QLabel(m_accentcolorwidget);
    title->setFixedWidth(m_accentcolorwidget->width());
    title->setText(tr("Accent colour"));
    QHBoxLayout *layout = new QHBoxLayout();
    m_combobox = new QComboBox(m_accentcolorwidget);
    m_combobox->addItem(tr("daybreakBlue"),QColor(55, 144, 250));
    m_combobox->addItem(tr("jamPurple"), QColor(120, 115, 245));
    m_combobox->addItem(tr("magenta"), QColor(235, 48, 150));
    m_combobox->addItem(tr("sunRed"), QColor(243, 34, 45));
    m_combobox->addItem(tr("sunsetOrange"), QColor(246, 140, 39));
    m_combobox->addItem(tr("dustGold"), QColor(249, 197, 61));
    m_combobox->addItem(tr("polarGreen"), QColor(82, 196, 41));

    connect(m_combobox, QOverload<int>::of(&QComboBox::currentIndexChanged), this, [=](int index){

        QColor selectedColor = m_combobox->itemData(index).value<QColor>();

        qDebug() << "Selected color:" << selectedColor;

        qDebug() << "RGB values:" << selectedColor.red() << selectedColor.green() << selectedColor.blue();
        m_preview->updatescale();
        m_preview->updateButtonColor(selectedColor);
        emit newAccentColor(selectedColor);
        g_themeChange = true;
    });

    layout->addWidget(title);
    layout->addWidget(m_combobox);

    m_accentcolorwidget->setLayout(layout);
}

/**
 * @brief 初始化透明度控件
 *
 * 创建一个QWidget作为透明度控件，并设置其固定高度。
 * 创建一个水平布局，添加标题标签和透明度滑块。
 * 设置滑块的相关属性，包括方向、固定宽度、范围、单步长、刻度间隔和固定宽度，并设置初始值为65。
 * 创建一个显示按钮，并设置其大小和文本。
 * 连接滑块的值变化信号与更新透明度的槽函数。
 * 设置透明度控件的布局。
 */
void GlobalThemeWidget::initTransparencyWidget()
{
    m_transparencywidget = new QWidget(m_viewportwidget);
    m_transparencywidget->setFixedHeight(65);
    QHBoxLayout *layout = new QHBoxLayout();
    QLabel *title = new QLabel(tr("Window transparency"));
    m_transparencyslider = new QSlider(m_filletwidget);

    m_transparencyslider->setOrientation(Qt::Horizontal);
    m_transparencyslider->setFixedWidth(210);
    m_transparencyslider->setRange(0,100);
    m_transparencyslider->setSingleStep(1);
    m_transparencyslider->setTickInterval(1);
    m_transparencyslider->setFixedWidth(120);
    m_transparencyslider->setValue(65);

    QPushButton *showButton = new QPushButton(m_filletwidget);
    showButton->setDisabled(true);
    showButton->setFixedSize(60,36);
    int value = 0;
    value = m_transparencyslider->value();
    showButton->setText(QString::number(m_transparencyslider->value())+"%");
    QObject::connect(m_transparencyslider, &QSlider::valueChanged, [=](int value) {
        showButton->setText(QString::number(value)+"%");
        newTransparency(value);
        m_preview->updateTransparency(value);
        g_themeChange = true;
    });
    layout->addWidget(title);
    layout->addWidget(m_transparencyslider);
    layout->addWidget(showButton);

    m_transparencywidget->setLayout(layout);
}

/**
 * @brief 初始化窗户圆角控件
 *
 * 创建一个新的 QWidget，并将其设置为视口小部件的父小部件。
 * 设置透明度小部件的固定高度为 65。
 * 创建一个水平布局并添加标签和下拉框。
 * 下拉框包含 "Large"、"Medium" 和 "Small" 三项，分别对应 12、6 和 0 的整数值。
 * 将下拉框的当前选项设置为 "Medium"，并连接下拉框的 currentIndexChanged 信号。
 * 当选项改变时，更新预览小部件的缩放比例，并更新控件的半径。
 * 发出新窗口半径信号，其中包含选定的半径大小。
 */
void GlobalThemeWidget::initWindowFilletWidget()
{
    m_windowfilletwidget = new QWidget(m_viewportwidget);
    m_transparencywidget->setFixedHeight(65);
    QHBoxLayout *layout = new QHBoxLayout();
    QLabel *title = new QLabel(tr("Window fillet"));

    m_filletcombobox = new QComboBox(m_windowfilletwidget);
    m_filletcombobox->addItem(tr("Large"), int(12));
    m_filletcombobox->addItem(tr("Medium"), int(6));
    m_filletcombobox->addItem(tr("Small"), int(0));
    m_filletcombobox->setCurrentIndex(1);
    connect(m_filletcombobox, QOverload<int>::of(&QComboBox::currentIndexChanged), this, [=](int index){
        // 获取当前选项对应的圆角大小值
        int selectedSize = m_filletcombobox->itemData(index).value<int>();
        // 更新预览部件的缩放比例
        m_preview->updatescale();
        // 更新预览部件的圆角大小
        m_preview->updateWidgetRadius(selectedSize);
        // 发出新的窗口圆角大小信号
        emit newWindowRadius(selectedSize);
        g_themeChange = true;
    });
    layout->addWidget(title);
    layout->addWidget(m_filletcombobox);
    m_windowfilletwidget->setLayout(layout);
}

/**
 * @brief 更新壁纸文件路径
 *
 * 根据传入的封面文件路径更新壁纸文件路径。
 *
 * @param coverFilePath 封面文件路径
 */
void GlobalThemeWidget::updateWallpaperFilePath(const QString& coverFilePath) {
    m_wallpaperpath = coverFilePath;

    if (m_preview != nullptr) {
        delete m_preview;
        delete m_previewlayout;

        m_previewwidget->setMinimumSize(495,620);
        m_previewwidget->setBackgroundRole(QPalette::Window);
        m_preview = new GlobalImageWidget(m_previewwidget,m_wallpaperpath);
        m_previewlayout = new QVBoxLayout(m_previewwidget);

        addspaceritem();

        m_previewwidget->setLayout(m_previewlayout);
    }
}

/**
 * @brief 添加间距项
 *
 * 在全局主题小部件中添加间距项。
 */
void GlobalThemeWidget::addspaceritem()
{
    QSpacerItem *spacer = new QSpacerItem(0, 0, QSizePolicy::Expanding, QSizePolicy::Expanding);
    QSpacerItem *spacer2 = new QSpacerItem(0, 0, QSizePolicy::Expanding, QSizePolicy::Expanding);
    spacer->changeSize(m_previewwidget->width(), 155, QSizePolicy::Maximum, QSizePolicy::Maximum);
    spacer2->changeSize(m_previewwidget->width(), 159, QSizePolicy::Maximum, QSizePolicy::Maximum);

    m_previewlayout->addItem(spacer);
    m_previewlayout->addWidget(m_preview);
    m_previewlayout->addItem(spacer2);
}

/**
 * @brief 设置圆角设置
 *
 * 将指定的圆角值设置到控件中。
 *
 * @param radius 圆角值
 */
void GlobalThemeWidget::setRadiusSetting(int radius)
{
    m_filletslider->setValue(radius);
}

/**
 * @brief 设置强调色
 *
 * 设置全局主题小部件的强调色。
 *
 * @param accentcolor 强调色的字符串表示形式
 */
void GlobalThemeWidget::setAccentColor(QString accentcolor)
{
    QColor color;
    color.setNamedColor(accentcolor);

    int index = m_combobox->findData(color);
    if (index != -1) {
        m_combobox->setCurrentIndex(index);
    }
}

/**
 * @brief 设置透明度
 *
 * 设置全局主题小部件的透明度。
 *
 * @param transparency 透明度值
 */
void GlobalThemeWidget::setTransparency(int transparency)
{
    // 设置透明度滑块的当前值
    m_transparencyslider->setValue(transparency);
}

/**
 * @brief 设置封面
 *
 * 设置封面按钮的图标为封面路径对应的图标。
 *
 * @param coverpath 封面路径
 */
void GlobalThemeWidget::setCover(QString coverpath)
{
    m_coverbtn->setIcon(QIcon(coverpath));
}

/**
 * @brief 设置壁纸
 *
 * 设置壁纸按钮的图标为壁纸路径对应的图标。
 *
 * @param wallpaperpath 壁纸路径
 */
void GlobalThemeWidget::setWallpaper(QString wallpaperpath)
{
    wallpaperbtn->setIcon(QIcon(wallpaperpath));
}

/**
 * @brief 设置窗口半径
 *
 * 根据给定的窗口半径，设置控件的圆角程度。
 *
 * @param windowradius 窗口半径
 */
void GlobalThemeWidget::setWindowRadius(int windowradius)
{
    QMap<int, int> radiusIndexMap;
    radiusIndexMap.insert(0, 2);
    radiusIndexMap.insert(6, 1);
    radiusIndexMap.insert(12, 0);

    if (radiusIndexMap.contains(windowradius)) {
        int index = radiusIndexMap.value(windowradius);
        m_filletcombobox->setCurrentIndex(index);
    }
}
