#include "historywidget.h"
#include "../fileProcess/historyinfoload.h"
#include "../fileProcess/diroperation.h"

/**
 * @brief 历史记录窗口
 *
 * 历史记录窗口类的构造函数，用于初始化历史记录窗口。
 *
 * @param parent 父窗口指针
 */
HistoryWidget::HistoryWidget(QWidget *parent) : QWidget(parent)
{
    QVBoxLayout *alllayout = new QVBoxLayout();

    QWidget *textWidget = new QWidget();
    QHBoxLayout *textlayout = new QHBoxLayout();
    QLabel *historyLabel = new QLabel(textWidget);
    historyLabel->setText(tr("Historical record"));
    QLabel *tipsLabel = new QLabel(textWidget);
    tipsLabel->setText(tr("Records are kept for 30 days only"));

    textlayout->addWidget(historyLabel);
    textlayout->addStretch(1);
    textlayout->addWidget(tipsLabel);
    textWidget->setLayout(textlayout);
    alllayout->addWidget(textWidget);
    QDir builderDir = QDir(QDir::homePath() + "/.cache/theme-build/");
    QStringList folders = builderDir.entryList(QDir::Dirs | QDir::NoDotAndDotDot);
    QList<QString> validFolders;

    QDateTime currentDate = QDateTime::currentDateTime();

    foreach (QString folder, folders) {
        QString yearStr = folder.mid(0, 4);
        QString monthStr = folder.mid(4, 2);
        QString dayStr = folder.mid(6, 2);
        QString hourStr = folder.mid(8, 2);
        QString minuteStr = folder.mid(10, 2);
        QString secondStr = folder.mid(12, 2);

        QDateTime folderDateTime = QDateTime::fromString(yearStr + monthStr + dayStr + hourStr + minuteStr + secondStr, "yyyyMMddhhmmss");

        if (folderDateTime.isValid() && folderDateTime.daysTo(currentDate) <= 30) {
            qDebug() << "Processing folder: " << folder;
            validFolders.append(folder);
        }
    }
    QGridLayout *layout = new QGridLayout;
    int row = 0;
    int col = 0;

    foreach (QString folder, validFolders) {
        HistoryButton *button = new HistoryButton(this);
        HistoryInfoLoad *info = new HistoryInfoLoad(folder);

        button->setHistoryTime(folder);
        button->setCover(info->getCover());
        button->setThemeName(info->getThemeName());
        button->setThemeType(info->getThemeType());
        button->initUI();
        connect(button,&HistoryButton::clicked,this,[=](){

            emit startSecondEdit(info->getInfoData());
        });
        layout->addWidget(button, row, col);
        col++;
        if (col == 4) {
            col = 0;
            row++;
        }
        connect(this,&HistoryWidget::updateInfo,info,&HistoryInfoLoad::updateHistoryInfo);
    }

    QWidget *widget = new QWidget(this);
    widget->setLayout(layout);
    alllayout->addWidget(widget);
    alllayout->addStretch(1);
    this->setLayout(alllayout);
}

/**
 * @brief 历史记录窗口
 *
 * 历史记录窗口类的构造函数，用于初始化历史记录窗口。
 *
 * @param parent 父窗口指针
 */
void HistoryWidget::updateHistoryDir()
{
    QDir builderDir = QDir(QDir::homePath() + "/.cache/theme-build/");
    QStringList folders = builderDir.entryList(QDir::Dirs | QDir::NoDotAndDotDot);
    QList<QString> validFolders;

    QDateTime currentDate = QDateTime::currentDateTime();

    foreach (QString folder, folders) {
        QString yearStr = folder.mid(0, 4);
        QString monthStr = folder.mid(4, 2);
        QString dayStr = folder.mid(6, 2);
        QString hourStr = folder.mid(8, 2);
        QString minuteStr = folder.mid(10, 2);
        QString secondStr = folder.mid(12, 2);

        QDateTime folderDateTime = QDateTime::fromString(yearStr + monthStr + dayStr + hourStr + minuteStr + secondStr, "yyyyMMddhhmmss");

        if (folderDateTime.isValid() && folderDateTime.daysTo(currentDate) <= 30) {
            qDebug() << "Processing folder222: " << folder;
            validFolders.append(folder);
        }
    }

    QGridLayout *layout = new QGridLayout;
    int row = 0;
    int col = 0;

    foreach (QString folder, validFolders) {
        HistoryButton *button = new HistoryButton(this);
        HistoryInfoLoad *info = new HistoryInfoLoad(folder);

        button->setHistoryTime(folder);
        button->setCover(info->getCover());
        button->setThemeName(info->getThemeName());
        button->setThemeType(info->getThemeType());
        button->initUI();
        connect(button,&HistoryButton::clicked,this,[=](){

            emit startSecondEdit(info->getInfoData());
        });
        layout->addWidget(button, row, col);

        col++;
        if (col == 4) {
            col = 0;
            row++;
        }
    }

    QWidget *widget = new QWidget(this);
    widget->setLayout(layout);

    QVBoxLayout *alllayout = qobject_cast<QVBoxLayout*>(this->layout());
    if (alllayout) {
        // 删除原来的widget
        while (QWidget* w = alllayout->takeAt(1)->widget()) {  // 从索引1开始，因为索引0是textWidget
            delete w;
        }
        alllayout->addWidget(widget);
        alllayout->addStretch(1);
    }
}

/**
 * @brief 历史按钮构造函数
 *
 * 创建一个历史按钮的构造函数。
 *
 * @param parent 父窗口指针
 */
HistoryButton::HistoryButton(QWidget* parent): QPushButton(parent)
{
//    this->setBackgroundRole(QPalette::Base);
    this->setFixedSize(240, 280);
    m_menu = new QMenu(this);
}

void HistoryButton::setHistoryTime(const QString &text)
{
    m_historytime = text;
}

void HistoryButton::setCover(const QString & covePath)
{
    m_coverpath = covePath;
}

void HistoryButton::setThemeName(const QString &themeName)
{
    m_themename = themeName;
}

void HistoryButton::setThemeType(const QString &themeType)
{
    m_themetype = themeType;
}

/**
 * @brief 初始化 UI
 *
 * 初始化 HistoryButton 的 UI。
 */
void HistoryButton::initUI()
{
    QVBoxLayout* layout = new QVBoxLayout(this);

    QLabel* imageLabel = new QLabel;
    imageLabel->setFixedSize(QSize(208, 137));
    layout->addWidget(imageLabel);

    QString imagePath = nullptr;
    if(nullptr == m_coverpath){
        imagePath = ":/resource/background/background-light.png";
    }else{
        imagePath = m_coverpath;
    };

    QImage image(imagePath);
    if (!image.isNull())
    {
        QImage scaledImage = image.scaled(QSize(208, 137), Qt::KeepAspectRatio);
        imageLabel->setPixmap(QPixmap::fromImage(scaledImage));
    }
    QLabel* label1 = new QLabel();
    if(m_themename != nullptr){
        label1->setText(m_themename);
    }else{
        label1->setText("未命名");
    }
    layout->addWidget(label1);

    QLabel* label2 = new QLabel(m_historytime);
    layout->addWidget(label2);

    QWidget * btnwidget = new QWidget(this);
    QHBoxLayout * btnwidgetlayout = new QHBoxLayout(btnwidget);

    QPushButton* button = new QPushButton(m_themetype);
    QPushButton* btn_menu = new QPushButton("···");
    btn_menu->setFixedWidth(50);
    btnwidgetlayout->addWidget(button);
    btnwidgetlayout->addStretch(1);
    btnwidgetlayout->addWidget(btn_menu);
    btnwidget->setLayout(btnwidgetlayout);
    layout->addWidget(btnwidget);

    this->setLayout(layout);
    // 创建并添加菜单选项
    QAction* action1 = m_menu->addAction("删除");
    QAction* action2 = m_menu->addAction("导出");

    connect(btn_menu, &QPushButton::clicked, this, [=]() {
        m_menu->exec(btn_menu->mapToGlobal(btn_menu->rect().bottomLeft()));
    });
    // 连接菜单选项的槽函数
    connect(action1, &QAction::triggered, this, [=]() {
        DirOperation *dop = new DirOperation();
        dop->DeleteHistoryDir(m_historytime);
        this->deleteLater();
//        emit updateHistoryRequested();
        this->parentWidget()->layout()->update();
    });


}


void HistoryWidget::handleButtonClick()
{

}
