#include "plymouththemefeature.h"
#include <QMovie>
#include <QLabel>
#include <QFileInfo>
#include <QMediaPlayer>
#include <QGraphicsVideoItem>
#include <QGraphicsProxyWidget>
#include <QMediaPlaylist>

PlymouthThemeFeature::PlymouthThemeFeature(QWidget *parent) : QWidget(parent)
{

}

/**
 * @brief PlymouthImageWidget 构造函数
 *
 * 创建一个 PlymouthImageWidget 对象，并将其设置为给定父对象的子对象。
 *
 * @param parent 父对象指针
 */
PlymouthImageWidget::PlymouthImageWidget(QWidget *parent)
{
    scene = new QGraphicsScene(this);
    graphicsView = new PlymouthGraphicsView(scene);

    layout = new QVBoxLayout(this);
    layout->addWidget(graphicsView);
    setLayout(layout);

    graphicsView->setScene(scene);

    // 显示gif
    gif_anim = new QLabel();
    QMovie *movie = new QMovie(":/resource/background/kylin.gif");
    gif_anim->setMovie(movie);
    movie->start();
    proxy = scene->addWidget(gif_anim);

    graphicsView->fitInView(scene->sceneRect(), Qt::KeepAspectRatio);
    graphicsView->show();

    qreal initialScale = 0.4; // 设置初始缩放倍数为0.4倍
    graphicsView->scale(initialScale, initialScale);
}

/**
 * @brief 更新 Plymouth 图像
 *
 * 根据给定的图像路径更新 Plymouth 图像。
 *
 * @param imagePath 图像路径
 */
void PlymouthImageWidget::updatePlymouth(const QString &imagePath)
{
    scene->clear(); // 清空当前场景

    QFileInfo fileInfo(imagePath);
    if (fileInfo.suffix().toLower() == "mp4") {
        QMediaPlayer *player = new QMediaPlayer;
        QMediaPlaylist *playlist = new QMediaPlaylist();
        playlist->addMedia(QUrl::fromLocalFile(imagePath));
        player->setMedia(QUrl::fromLocalFile(imagePath));
        playlist->setPlaybackMode(QMediaPlaylist::Loop);
        player->setPlaylist(playlist);
        player->setMuted(true);
        QGraphicsVideoItem *videoItem = new QGraphicsVideoItem;
        videoItem->setSize(scene->sceneRect().size()); // 设置视频项的大小以匹配场景
        videoItem->setPos(0, 0); // 设置视频项的位置

        scene->addItem(videoItem); // 将视频项添加到场景中
        player->setVideoOutput(videoItem); // 为播放器设置视频输出
        player->play();
    } else {
        QPixmap pixmap(imagePath);
        QGraphicsPixmapItem *pixmapItem = new QGraphicsPixmapItem(pixmap);
        scene->addItem(pixmapItem);

        // 在场景中居中显示图像项
        qreal x = (scene->width() - pixmap.width()) / 2;
        qreal y = (scene->height() - pixmap.height()) / 2;
        pixmapItem->setPos(x, y);
    }

    graphicsView->fitInView(scene->sceneRect(), Qt::KeepAspectRatio); // 调整视图以适应新内容
}

/**
 * @brief 显示 Plymouth 图像
 *
 * 通过定时器，依次显示 pixmaps 中的图像，形成动画效果。
 */
PlymouthGraphicsView::PlymouthGraphicsView(QGraphicsScene *scene)
{
    setTransformationAnchor(QGraphicsView::AnchorUnderMouse);
    setDragMode(QGraphicsView::ScrollHandDrag);
    setRenderHint(QPainter::Antialiasing, true);
    setOptimizationFlags(QGraphicsView::DontAdjustForAntialiasing |
                         QGraphicsView::DontSavePainterState);
    this->setStyleSheet("background-color: transparent;");
    setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
}

void PlymouthGraphicsView::wheelEvent(QWheelEvent *event)
{
    if (QGuiApplication::keyboardModifiers() == Qt::ControlModifier) {
        qreal scaleFactor = std::pow(qreal(2), event->delta() / 240.0);
        qreal currentScale = transform().m11(); // 获取当前的缩放倍数
//        // 限制最大为1.1倍
//        qreal targetScale = currentScale * scaleFactor;
//        if (targetScale > 1.1) {
//            scaleFactor = 1.1 / currentScale;
//        }else if (targetScale < 0.3) {
//            scaleFactor = 0.3 / currentScale;
//        }

        scale(scaleFactor, scaleFactor);
    } else {
        QGraphicsView::wheelEvent(event);
    }
}

void PlymouthGraphicsView::drawBackground(QPainter *painter, const QRectF &rect)
{
    // 绘制全黑背景
    painter->fillRect(rect, Qt::black);
}
