#include "plymouththemewidget.h"

bool PlymouthThemeWidget::g_themeChange;
/**
 * @brief PlymouthThemeWidget 构造函数
 *
 * 创建一个 PlymouthThemeWidget 对象，并设置其父对象。
 *
 * @param parent 父对象
 */
PlymouthThemeWidget::PlymouthThemeWidget(QWidget *parent) : QWidget(parent)
{
    initPreviewWidget();
    initRightWidget();
    QHBoxLayout *globalthemelayout = new QHBoxLayout();
    globalthemelayout->addWidget(m_previewwidget);
    globalthemelayout->addWidget(m_rightwidget);
    this->setLayout(globalthemelayout);
}

/**
 * @brief 初始化预览小部件
 *
 * 创建一个新的 QWidget 作为预览小部件，并设置其最小大小。
 * 设置预览小部件的背景角色为 QPalette::Window。
 * 设置预览小部件的调色板，并开启自动填充背景。
 * 创建一个新的 PlymouthImageWidget，并将其添加到预览小部件的垂直布局中。
 * 添加一个空隙项，并设置预览小部件的布局。
 */
void PlymouthThemeWidget::initPreviewWidget()
{
    m_previewwidget = new QWidget(this);
    m_previewwidget->setMinimumSize(495,620);

    m_previewwidget->setBackgroundRole(QPalette::Window);

    QPalette palette;
    QColor bgColor("#F5F5F5");
    palette.setColor(QPalette::Window, bgColor);
    m_previewwidget->setPalette(palette);
    m_previewwidget->setAutoFillBackground(true);

    m_preview = new PlymouthImageWidget(m_previewwidget);

    m_previewlayout = new QVBoxLayout(m_previewwidget);
    addspaceritem();
    m_previewwidget->setLayout(m_previewlayout);
}

/**
 * @brief 初始化右侧小部件
 *
 * 创建一个新的QWidget对象作为右侧小部件，并设置其最小和最大尺寸，以及背景色。
 * 之后初始化编辑小部件。
 */
void PlymouthThemeWidget::initRightWidget()
{
    m_rightwidget = new QWidget(this);
    m_rightwidget->setMinimumSize(400,620);
    m_rightwidget->setMaximumWidth(400);
    m_rightwidget->setBackgroundRole(QPalette::Base);
    initEditWidget();
}

/**
 * @brief 初始化编辑小部件
 *
 * 初始化编辑小部件，包括设置固定高度、创建布局、添加控件等。
 */
void PlymouthThemeWidget::initEditWidget()
{
    m_plymouthwidget = new QWidget(m_rightwidget);
    m_plymouthwidget->setFixedHeight(150);
    QVBoxLayout *widgetLayout = new QVBoxLayout(m_plymouthwidget);
    QWidget *checkBoxWidget = new QWidget();
    QHBoxLayout *checkBoxLayout = new QHBoxLayout(checkBoxWidget);
    QCheckBox *pic = new QCheckBox();
    QLabel *pixLabel = new QLabel("图片");
    QCheckBox *video = new QCheckBox();
    QLabel *videoLabel = new QLabel("视频");
    checkBoxLayout->addWidget(pic);
    checkBoxLayout->addWidget(pixLabel);
    checkBoxLayout->addWidget(video);
    checkBoxLayout->addWidget(videoLabel);
    checkBoxLayout->addStretch(1);
    QWidget *showWidget = new QWidget();
    QHBoxLayout *showLayout = new QHBoxLayout(showWidget);

    QLabel *defaultLabel = new QLabel();
    defaultLabel->setFixedSize(64,64);
    QPixmap pixmap(":/resource/background/openKylin.svg");
    pixmap = pixmap.scaled(64,64, Qt::KeepAspectRatio);
    defaultLabel->setPixmap(pixmap);
    m_customLabel= new CustomLabel();
    m_customLabel->setFixedSize(64,64);
    QPushButton *addButton = new QPushButton();
    addButton->setIcon(QIcon::fromTheme("list-add-symbolic"));
    addButton->setFixedSize(36,36);
    connect(addButton, &QPushButton::clicked, this, [=]() {
        QString newFilePath = QFileDialog::getOpenFileName(this, tr("Select MP4 file"), "", tr("MP4 file (*.svg *.png *.gif *.mp4)"));
        if (!newFilePath.isEmpty()) {
            QPixmap pixmap(newFilePath);
            pixmap = pixmap.scaled(64,64, Qt::KeepAspectRatio);
            m_customLabel->setPixmap(pixmap);

            m_preview->updatePlymouth(newFilePath);
            emit newPlymouthFilePath(newFilePath);
            g_themeChange = true;
        }

    });
    QLabel *tipLabel = new QLabel();
    tipLabel->setText("<html>大小不超过10MB<br>格式：MP4</html>");

    showLayout->addWidget(defaultLabel);
    showLayout->addWidget(m_customLabel);
    showLayout->addWidget(addButton);
    showLayout->addStretch(1);
    showLayout->addWidget(tipLabel);

    showWidget->setLayout(showLayout);

    widgetLayout->addWidget(checkBoxWidget);
    widgetLayout->addWidget(showWidget);
    m_plymouthwidget->setLayout(widgetLayout);
}

void PlymouthThemeWidget::setThumbnailIcon(QString path)
{
    QPixmap pixmap(path);
    pixmap = pixmap.scaled(64,64, Qt::KeepAspectRatio);
    m_customLabel->setPixmap(pixmap);
}

void PlymouthThemeWidget::addspaceritem()
{
    QSpacerItem *spacer = new QSpacerItem(0, 0, QSizePolicy::Expanding, QSizePolicy::Expanding);
    QSpacerItem *spacer2 = new QSpacerItem(0, 0, QSizePolicy::Expanding, QSizePolicy::Expanding);
    spacer->changeSize(m_previewwidget->width(), 155, QSizePolicy::Maximum, QSizePolicy::Maximum);
    spacer2->changeSize(m_previewwidget->width(), 159, QSizePolicy::Maximum, QSizePolicy::Maximum);

    m_previewlayout->addItem(spacer);
    m_previewlayout->addWidget(m_preview);
    m_previewlayout->addItem(spacer2);
}

