#include "titlebar.h"

/**
 * @brief TitleBar类的构造函数
 *
 * TitleBar类的构造函数，用于创建一个带有标题栏的窗口。
 *
 * @param parent 父窗口指针
 */
TitleBar::TitleBar(QWidget *parent) : QWidget(parent)
{
    m_ismaximized = false;
    this->setFixedHeight(30);

    QVBoxLayout *layout = new QVBoxLayout(this);
    layout->setMargin(0);
    layout->setSpacing(0);

    QLabel *iconlabel = new QLabel(this);
    iconlabel->setFixedSize(30, 30);
    QIcon icon = QIcon::fromTheme("viewnior");
    QIcon scaledIcon = icon.pixmap(QSize(30, 30)).scaled(QSize(30, 30), Qt::KeepAspectRatio, Qt::SmoothTransformation);
    QPixmap pixmap = scaledIcon.pixmap(QSize(30, 30));
    iconlabel->setPixmap(pixmap);

    QLabel *title = new QLabel("主题制作工具");

    m_gohomebtn = new QPushButton();
    m_gohomebtn->setFixedSize(30,30);
    m_gohomebtn->setIcon(QIcon::fromTheme("go-home-symbolic"));
    m_gohomebtn->setFlat(true);

    m_optionbtn = new QToolButton();
    m_optionbtn->setToolTip(tr("Menu"));
    m_optionbtn->setPopupMode(QToolButton::InstantPopup);
    m_optionbtn->setFixedSize(30,30);
    m_optionbtn->setIcon(QIcon::fromTheme("application-menu"));
    m_optionbtn->setAutoRaise(true);

    QMenu* menu = new QMenu(m_optionbtn);

    QAction* saveAction = new QAction(tr("Save path"), menu);
    menu->addAction(saveAction);
    QAction* aboutAction = new QAction(tr("about"), menu);
    menu->addAction(aboutAction);
    QAction* helpAction = new QAction(tr("help"), menu);
    menu->addAction(helpAction);
    QAction* exitAction = new QAction(tr("exit"), menu);
    menu->addAction(exitAction);
    m_optionbtn->setMenu(menu);

    m_minimumbtn = new QPushButton();
    m_minimumbtn->setFixedSize(30,30);
    m_minimumbtn->setIcon(QIcon::fromTheme("window-minimize-symbolic"));
    m_minimumbtn->setFlat(true);

    m_maximumbtn = new QPushButton();
    m_maximumbtn->setFixedSize(30,30);
    m_maximumbtn->setIcon(QIcon::fromTheme("window-maximize-symbolic"));
    m_maximumbtn->setFlat(true);

    m_closebtn = new QPushButton();
    m_closebtn->setFixedSize(30, 30);
    m_closebtn->setIcon(QIcon::fromTheme("window-close-symbolic"));
    m_closebtn->setFlat(true);

    connect(m_gohomebtn, &QPushButton::clicked, this,[=](){
        emit gohomesignal();
    });
    connect(m_minimumbtn, &QPushButton::clicked, parent, &QWidget::showMinimized);
    connect(m_maximumbtn, &QPushButton::clicked, this, &TitleBar::toggleMaximize);
    connect(m_closebtn, &QPushButton::clicked, parent, &QWidget::close);

    layout->addStretch();

    QHBoxLayout *buttonLayout = new QHBoxLayout();
    buttonLayout->addWidget(iconlabel);
    buttonLayout->addWidget(title);
    buttonLayout->addStretch();
    buttonLayout->addWidget(m_gohomebtn);
    buttonLayout->addWidget(m_optionbtn);
    buttonLayout->addWidget(m_minimumbtn);
    buttonLayout->addWidget(m_maximumbtn);
    buttonLayout->addWidget(m_closebtn);


    layout->addLayout(buttonLayout);
}

/**
 * @brief 切换最大化状态
 *
 * 如果当前窗口不是最大化状态，则将其最大化，并更新相应的按钮图标。
 * 如果当前窗口已经是最大化状态，则将其恢复为正常状态，并更新相应的按钮图标。
 */
void TitleBar::toggleMaximize()
{
    // 如果当前窗口不是最大化状态
    if (!m_ismaximized) {
        // 显示最大化窗口
        parentWidget()->parentWidget()->showMaximized();
        // 设置状态为已最大化
        m_ismaximized = true;
        // 设置最大化按钮的图标为还原图标
        m_maximumbtn->setIcon(QIcon::fromTheme("window-restore-symbolic"));

    // 如果当前窗口已经是最大化状态
    } else {
        // 显示普通窗口
        parentWidget()->parentWidget()->showNormal();
        // 设置状态为未最大化
        m_ismaximized = false;
        // 设置最大化按钮的图标为最大化图标
        m_maximumbtn->setIcon(QIcon::fromTheme("window-maximize-symbolic"));
    }
}
